#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function, division, unicode_literals
import unittest
import os
import datetime

from zhihu import Question, Activity, ActType
from zhihu.common import BeautifulSoup

from test_utils import TEST_DATA_PATH


class ActivityTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        url = 'http://www.zhihu.com/question/24825703'
        file_path = os.path.join(TEST_DATA_PATH, 'question.html')
        with open(file_path, 'rb') as f:
            html = f.read()
        soup = BeautifulSoup(html)

        cls.question = Question(url)
        cls.question._session = None
        cls.question.soup = soup

        act_time = datetime.datetime.fromtimestamp(1439395600)
        act_type = ActType.FOLLOW_QUESTION
        cls.activity = Activity(act_type, act_time, question=cls.question)

    def test_content(self):
        self.assertIs(self.question, self.activity.content)

    def test_init_errors(self):
        act_time = datetime.datetime.fromtimestamp(1439395600)
        act_type = ActType.FOLLOW_QUESTION

        with self.assertRaises(ValueError):
            Activity(100, act_time)
        with self.assertRaises(ValueError):
            Activity(act_type, act_time)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import *


class Collection:

    """收藏夹，请使用``ZhihuClient.collection``方法构造对象."""

    @class_common_init(re_collection_url)
    def __init__(self, url, owner=None, name=None, follower_num=None,
                 session=None):
        """创建收藏夹类实例.

        :param str url: 收藏夹主页url，必须
        :param Author owner: 收藏夹拥有者，可选
        :param str name: 收藏夹标题，可选
        :param int follower_num: 收藏夹关注人数，可选
        :param Session session: 使用的网络会话，为空则使用新会话。
        :return: 收藏夹对象
        :rtype: Collection
        """
        self.url = url
        self._session = session
        self.soup = None
        self._name = name
        self._owner = owner
        self._follower_num = follower_num

    def _make_soup(self):
        if self.soup is None:
            self.soup = BeautifulSoup(self._session.get(self.url).text)

    @property
    @check_soup('_cid')
    def cid(self):
        """获取收藏夹内部Id（用不到忽视就好）

        :return: 内部Id
        :rtype: int
        """
        return int(re_get_number.match(
            self.soup.find('a', attrs={'name': 'focus'})['id']).group(1))

    @property
    @check_soup('_xsrf')
    def xsrf(self):
        """获取知乎的反xsrf参数（用不到就忽视吧~）

        :return: xsrf参数
        :rtype: str
        """
        return self.soup.find(
            'input', attrs={'name': '_xsrf'})['value']

    @property
    @check_soup('_name')
    def name(self):
        """获取收藏夹名字.

        :return: 收藏夹名字
        :rtype: str
        """
        return re_del_empty_line.match(
            self.soup.find('h2', id='zh-fav-head-title').text).group(1)

    @property
    @check_soup('_owner')
    def owner(self):
        """获取收藏夹拥有者，返回Author对象.

        :return: 收藏夹拥有者
        :rtype: Author
        """
        from .author import Author

        a = self.soup.find('h2', class_='zm-list-content-title').a
        name = a.text
        url = Zhihu_URL + a['href']
        motto = self.soup.find(
            'div', id='zh-single-answer-author-info').div.text
        photo_url = PROTOCOL + self.soup.find(
            'img', class_='zm-list-avatar-medium')['src'].replace('_m', '_r')
        return Author(url, name, motto, photo_url=photo_url,
                      session=self._session)

    @property
    @check_soup('_follower_num')
    def follower_num(self):
        """获取关注此收藏夹的人数.

        :return: 关注此收藏夹的人数
        :rtype: int
        """
        href = re_collection_url_split.match(self.url).group(1)
        return int(self.soup.find('a', href=href + 'followers').text)

    @property
    def followers(self):
        """获取关注此收藏夹的用户

        :return: 关注此收藏夹的用户
        :rtype: Author.Iterable
        """
        self._make_soup()
        followers_url = self.url + 'followers'
        for x in common_follower(followers_url, self.xsrf, self._session):
            yield x

    @property
    def questions(self):
        """获取收藏夹内所有问题对象.

        :return: 收藏夹内所有问题，返回生成器
        :rtype: Question.Iterable
        """
        self._make_soup()
        # noinspection PyTypeChecker
        for question in self._page_get_questions(self.soup):
            yield question
        i = 2
        while True:
            soup = BeautifulSoup(self._session.get(
                self.url[:-1] + '?page=' + str(i)).text)
            for question in self._page_get_questions(soup):
                if question == 0:
                    return
                yield question
            i += 1

    @property
    def answers(self):
        """获取收藏夹内所有答案对象.

        :return: 收藏夹内所有答案，返回生成器
        :rtype: Answer.Iterable
        """
        self._make_soup()
        # noinspection PyTypeChecker
        for answer in self._page_get_answers(self.soup):
            yield answer
        i = 2
        while True:
            soup = BeautifulSoup(self._session.get(
                self.url[:-1] + '?page=' + str(i)).text)
            for answer in self._page_get_answers(soup):
                if answer == 0:
                    return
                yield answer
            i += 1

    def _page_get_questions(self, soup):
        from .question import Question

        question_tags = soup.find_all("div", class_="zm-item")
        if len(question_tags) == 0:
            yield 0
            return
        else:
            for question_tag in question_tags:
                if question_tag.h2 is not None:
                    question_title = question_tag.h2.a.text
                    question_url = Zhihu_URL + question_tag.h2.a['href']
                    yield Question(question_url, question_title,
                                   session=self._session)

    def _page_get_answers(self, soup):
        from .question import Question
        from .author import Author
        from .answer import Answer

        answer_tags = soup.find_all("div", class_="zm-item")
        if len(answer_tags) == 0:
            yield 0
            return
        else:
            question = None
            for tag in answer_tags:
                # 判断是否是'建议修改的回答'等情况
                url_tag = tag.find('a', class_='answer-date-link')
                if url_tag is None:
                    reason = tag.find('div', id='answer-status').p.text
                    print("pass a answer, reason %s ." % reason)
                    continue
                author_name = '匿名用户'
                author_motto = ''
                author_url = None
                if tag.h2 is not None:
                    question_title = tag.h2.a.text
                    question_url = Zhihu_URL + tag.h2.a['href']
                    question = Question(question_url, question_title,
                                        session=self._session)
                answer_url = Zhihu_URL + url_tag['href']
                h3 = tag.find('h3')
                if h3.text != '匿名用户':
                    author_url = Zhihu_URL + h3.a['href']
                    author_name = h3.a.text
                    if h3.strong is not None:
                        author_motto = tag.find('h3').strong['title']
                print(author_url)
                author = Author(author_url, author_name, author_motto,
                                session=self._session)
                upvote = int(tag.find(
                    'a', class_='zm-item-vote-count')['data-votecount'])
                answer = Answer(answer_url, question, author,
                                upvote, session=self._session)
                yield answer



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


class Activity:

    """用户动态类，请使用Author.activities获取."""

    def __init__(self, act_type, act_time, **kwarg):
        """创建用户动态类实例.

        :param ActType act_type: 动态类型
        :param datatime.datatime act_time: 动态发生时间
        :return: 用户动态对象
        :rtype: Activity

        :说明:
            根据Activity.type的不同可以获取的属性，具体请看 :class:`.ActType`

        """
        from .acttype import ActType

        if not isinstance(act_type, ActType):
            raise ValueError('invalid activity type')
        if len(kwarg) != 1:
            raise ValueError('except one kwarg (%d given)' % len(kwarg))
        self.type = act_type
        self.time = act_time
        for k, v in kwarg.items():
            self._attr = k
            setattr(self, k, v)

    @property
    def content(self):
        """获取此对象中能提供的那个属性，对应表请查看ActType类.

        :return: 对象提供的对象
        :rtype: Author or Question or Answer or Topic or Column or Post
        """
        return getattr(self, self._attr)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function, division, unicode_literals
import unittest
import os

from zhihu import Answer
from zhihu.common import BeautifulSoup
from test_utils import TEST_DATA_PATH


class AnswerTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        url = 'http://www.zhihu.com/question/24825703/answer/30975949'
        file_path = os.path.join(TEST_DATA_PATH, 'answer.html')
        with open(file_path, 'rb') as f:
            html = f.read()
        soup = BeautifulSoup(html)

        answer_saved_path = os.path.join(TEST_DATA_PATH, 'answer.md')
        with open(answer_saved_path, 'rb') as f:
            cls.answer_saved = f.read()

        cls.answer = Answer(url)
        cls.answer._session = None
        cls.answer.soup = soup
        cls.expected = {'id': 30975949, 'aid': 7775236,
                        'xsrf': 'cfd489623d34ca03adfdc125368c6426',
                        'html': soup.prettify(), 'author_id': 'tian-ge-xia',
                        'author_name': '甜阁下', 'question_id': 24825703,
                        'question_title': '关系亲密的人之间要说「谢谢」吗？',
                        'upvote_num': 1164, 'upvoter_name': 'Mikuroneko',
                        'upvoter_id': 'guo-yi-hui-23'}

    def test_id(self):
        self.assertEqual(self.expected['id'], self.answer.id)

    def test_aid(self):
        self.assertEqual(self.expected['aid'], self.answer.aid)

    def test_xsrf(self):
        self.assertEqual(self.expected['xsrf'], self.answer.xsrf)

    def test_html(self):
        self.assertEqual(self.expected['html'], self.answer.html)

    def test_upvote_num(self):
        self.assertEqual(self.expected['upvote_num'], self.answer.upvote_num)

    def test_author(self):
        self.assertEqual(self.expected['author_id'], self.answer.author.id)
        self.assertEqual(self.expected['author_name'], self.answer.author.name)

    def test_question(self):
        self.assertEqual(self.expected['question_id'], self.answer.question.id)
        self.assertEqual(self.expected['question_title'],
                         self.answer.question.title)

    def test_content(self):
        path = os.path.join(TEST_DATA_PATH, 'answer_content.html')
        with open(path, 'rb') as f:
            content = f.read()
        self.assertEqual(content.decode('utf-8'), self.answer.content)

    def test_save(self):
        save_name = 'answer_save'
        self.answer.save(filepath=TEST_DATA_PATH, filename=save_name,
                         mode='md')
        answer_saved_path = os.path.join(TEST_DATA_PATH, save_name + '.md')
        with open(answer_saved_path, 'rb') as f:
            answer_saved = f.read()
        os.remove(answer_saved_path)
        self.assertEqual(self.answer_saved, answer_saved)

    def test_parse_author_soup(self):
        fpath = os.path.join(TEST_DATA_PATH, 'answer_upvoter.html')
        with open(fpath, 'rb') as f:
            html = f.read().decode('utf-8')

        soup = BeautifulSoup(html)
        upvoter = self.answer._parse_author_soup(soup)

        self.assertEqual(self.expected['upvoter_name'], upvoter.name)
        self.assertEqual(self.expected['upvoter_id'], upvoter.id)

    def test_save_error(self):
        with self.assertRaises(ValueError):
            self.answer.save(filepath=TEST_DATA_PATH, filename='invalid',
                             mode='invalid')



import os
import inspect


def module_path(local_function):
    ''' returns the module path without the use of __file__.
    Requires a function defined locally in the module.
    from "http://stackoverflow.com/questions/729583/
    getting-file-path-of-imported-module"'''

    return os.path.abspath(inspect.getsourcefile(local_function))


TEST_DATA_PATH = os.path.join(
    os.path.split(module_path(module_path))[0], 'data')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import *


class Column:

    """专栏类，请使用``ZhihuClient.column``方法构造对象."""

    @class_common_init(re_column_url)
    def __init__(self, url, name=None, follower_num=None,
                 post_num=None, session=None):
        """创建专栏类实例.

        :param str url: 专栏url
        :param str name: 专栏名，可选
        :param int follower_num: 关注者数量，可选
        :param int post_num: 文章数量，可选
        :param Session session: 使用的网络会话，为空则使用新会话。
        :return: 专栏对象
        :rtype: Column
        """
        self._in_name = re_column_url.match(url).group(1)
        self.url = url
        self._session = session
        self._name = name
        self._follower_num = follower_num
        self._post_num = post_num

    def _make_soup(self):
        if self.soup is None:
            origin_host = self._session.headers.get('Host')
            self._session.headers.update(Host='zhuanlan.zhihu.com')
            res = self._session.get(Column_Data.format(self._in_name))
            self._session.headers.update(Host=origin_host)
            self.soup = res.json()

    @property
    @check_soup('_name')
    def name(self):
        """获取专栏名称.

        :return: 专栏名称
        :rtype: str
        """
        return self.soup['name']

    @property
    @check_soup('_follower_num')
    def follower_num(self):
        """获取关注人数.

        :return: 关注人数
        :rtype: int
        """
        return int(self.soup['followersCount'])

    @property
    @check_soup('_post_num')
    def post_num(self):
        """获取专栏文章数.

        :return: 专栏文章数
        :rtype: int
        """
        return int(self.soup['postsCount'])

    @property
    def posts(self):
        """获取专栏的所有文章.

        :return: 专栏所有文章，返回生成器
        :rtype: Post.Iterable
        """
        origin_host = self._session.headers.get('Host')
        for offset in range(0, (self.post_num - 1) // 10 + 1):
            self._session.headers.update(Host='zhuanlan.zhihu.com')
            res = self._session.get(
                Column_Posts_Data.format(self._in_name, offset * 10))
            soup = res.json()
            self._session.headers.update(Host=origin_host)
            for post in soup:
                yield self._parse_post_data(post)

    def _parse_post_data(self, post):
        from .author import Author
        from .post import Post

        url = Column_Url + post['url']
        template = post['author']['avatar']['template']
        photo_id = post['author']['avatar']['id']
        photo_url = template.format(id=photo_id, size='r')
        author = Author(post['author']['profileUrl'],
                        post['author']['name'], post['author']['bio'],
                        photo_url=photo_url, session=self._session)
        title = post['title']
        upvote_num = post['likesCount']
        comment_num = post['commentsCount']
        print(url)
        return Post(url, self, author, title, upvote_num, comment_num,
                    session=self._session)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = '7sDream'

import os
import shutil
from zhihu import ZhihuClient, ActType


def test_question():
    url = 'http://www.zhihu.com/question/24825703'
    question = client.question(url)

    # 获取该问题的详细描述
    print(question.title)
    # 亲密关系之间要说「谢谢」吗？

    # 获取该问题的详细描述
    print(question.details)
    # 从小父母和大家庭里，.......什么时候不该说"谢谢”？？

    # 获取回答个数
    print(question.answer_num)
    # 630

    # 获取关注该问题的人数
    print(question.follower_num)
    # 4326

    # 获取关注问题的用户
    for _, follower in zip(range(0, 10), question.followers):
        print(follower.name)
    # J Drop
    # 熊猫
    # Steve He
    # ...

    # 获取该问题所属话题
    print(question.topics)
    # ['心理学', '恋爱', '社会', '礼仪', '亲密关系']

    # 获取排名第一的回答的点赞数
    print(question.top_answer.upvote_num)
    # 197

    # 获取排名前十的十个回答的点赞数
    for answer in question.top_i_answers(10):
        print(answer.author.name, answer.upvote_num)
    # 49
    # 89
    # 425
    # ...


def test_answer():
    url = 'http://www.zhihu.com/question/24825703/answer/30975949'
    answer = client.answer(url)

    # 获取答案url
    print(answer.url)

    # 获取该答案所在问题标题
    print(answer.question.title)
    # 关系亲密的人之间要说「谢谢」吗？

    # 获取该答案作者名
    print(answer.author.name)
    # 甜阁下

    # 获取答案赞同数
    print(answer.upvote_num)
    # 1155

    # 获取答案点赞人昵称和感谢赞同提问回答数
    for _, upvoter in zip(range(1, 10), answer.upvoters):
        print(upvoter.name, upvoter.upvote_num, upvoter.thank_num,
              upvoter.question_num, upvoter.answer_num, upvoter.is_zero_user())
    # ...
    # 五月 42 15 3 35
    # 陈半边 6311 1037 3 101
    # 刘柯 3107 969 273 36
    #
    # 三零用户比例 36.364%

    # 获取答案内容的HTML
    print(answer.content)
    # <html>
    # ...
    # </html>

    # 保存HTML
    answer.save(filepath='.')
    # 当前目录下生成 "亲密关系之间要说「谢谢」吗？ - 甜阁下.html"

    # 保存markdown
    answer.save(filepath='.', mode="md")
    # 当前目录下生成 "亲密关系之间要说「谢谢」吗？ - 甜阁下.md"


def test_author():
    url = 'http://www.zhihu.com/people/7sdream'
    author = client.author(url)

    # 获取用户名称
    print(author.name)
    # 7sDream

    # 获取用户介绍
    print(author.motto)
    # 二次元新居民/软件爱好者/零回答消灭者

    # 获取用户头像地址
    print(author.photo_url)
    # http://pic3.zhimg.com/893fd554c8aa57196d5ab98530ef479a_r.jpg

    # 获取用户得到赞同数
    print(author.upvote_num)
    # 1338

    # 获取用户得到感谢数
    print(author.thank_num)
    # 468

    # 获取用户关注人数
    print(author.followee_num)
    # 82

    # 获取用户关注人
    for _, followee in zip(range(0, 10), author.followees):
        print(followee.name)
    # yuwei
    # falling
    # 伍声
    # bhuztez
    # 段晓晨
    # 冯东
    # ...

    # 获取用户粉丝数
    print(author.follower_num)
    # 303

    # 获得用户粉丝
    for _, follower in zip(range(0, 10), author.followers):
        print(follower.name)
    # yuwei
    # falling
    # 周非
    # 陈泓瑾
    # O1Operator
    # ...

    # 获取用户提问数
    print(author.question_num)
    # 16

    # 获取用户所有提问的标题
    for _, question in zip(range(0, 10), author.questions):
        print(question.title)
    # 用户「松阳先生」的主页出了什么问题？
    # C++运算符重载在头文件中应该如何定义？
    # 亚马逊应用市场的应用都是正版的吗？
    # ...

    # 获取用户答题数
    print(author.answer_num)
    # 247

    # 获取用户所有回答的点赞数
    for _, answer in zip(range(0, 10), author.answers):
        print(answer.upvote_num)
    # 0
    # 5
    # 12
    # 0
    # ...

    # 获取用户文章数
    print(author.post_num)
    # 0

    # 获取用户专栏名
    for column in author.columns:
        print(column.name)
    # 我没有专栏T^T

    # 获取用户收藏夹数
    print(author.collection_num)
    # 5

    # 获取用户收藏夹名
    for collection in author.collections:
        print(collection.name)
    # 教学精品。
    # 可以留着慢慢看～
    # OwO
    # 一句。
    # Read it later

    # 获取用户关注专栏数
    print(author.followed_column_num)
    # 9

    # 获取用户关注的专栏
    for column in author.followed_columns:
        print(column.name, column.url, column.follower_num)

    # 获取用户动态
    for _, act in zip(range(0, 10), author.activities):
        print(act.content.url)
        if act.type == ActType.FOLLOW_COLUMN:
            print('%s 在 %s 关注了专栏 %s' %
                  (author.name, act.time, act.column.name))
        elif act.type == ActType.FOLLOW_QUESTION:
            print('%s 在 %s 关注了问题 %s' %
                  (author.name, act.time, act.question.title))
        elif act.type == ActType.ASK_QUESTION:
            print('%s 在 %s 提了个问题 %s' %
                  (author.name, act.time, act.question.title))
        elif act.type == ActType.UPVOTE_POST:
            print('%s 在 %s 赞同了专栏 %s 中 %s 的文章 %s, '
                  '此文章赞同数 %d, 评论数 %d' %
                  (author.name, act.time, act.post.column.name,
                   act.post.author.name, act.post.title, act.post.upvote_num,
                   act.post.comment_num))
        elif act.type == ActType.PUBLISH_POST:
            print('%s 在 %s 在专栏 %s 中发布了文章 %s, '
                  '此文章赞同数 %d, 评论数 %d' %
                  (author.name, act.time, act.post.column.name,
                   act.post.title, act.post.upvote_num,
                   act.post.comment_num))
        elif act.type == ActType.UPVOTE_ANSWER:
            print('%s 在 %s 赞同了问题 %s 中 %s(motto: %s) 的回答, '
                  '此回答赞同数 %d' %
                  (author.name, act.time, act.answer.question.title,
                   act.answer.author.name, act.answer.author.motto,
                   act.answer.upvote_num))
        elif act.type == ActType.ANSWER_QUESTION:
            print('%s 在 %s 回答了问题 %s 此回答赞同数 %d' %
                  (author.name, act.time, act.answer.question.title,
                   act.answer.upvote_num))
        elif act.type == ActType.FOLLOW_TOPIC:
            print('%s 在 %s 关注了话题 %s' %
                  (author.name, act.time, act.topic.name))


def test_collection():
    url = 'http://www.zhihu.com/collection/28698204'
    collection = client.collection(url)

    # 获取收藏夹名字
    print(collection.name)
    # 可以用来背的答案

    # 获取收藏夹关注人数
    print(collection.follower_num)
    # 6343

    # 获取收藏夹关注用户
    for _, follower in zip(range(0, 10), collection.followers):
        print(follower.name)
    # 花椰菜
    # 邱火羽白
    # 枫丹白露
    # ...

    # 获取收藏夹创建者名字
    print(collection.owner.name)
    # 7sDream

    # 获取收藏夹内所有答案的点赞数
    for _, answer in zip(range(0, 10), collection.answers):
        print(answer.upvote_num)
    # 2561
    # 535
    # 223
    # ...

    # 获取收藏夹内所有问题标题
    for _, question in zip(range(0, 10), collection.questions):
        print(question.title)
    # 如何完成标准的平板支撑？
    # 有没有适合 Android 开发初学者的 App 源码推荐？
    # 如何挑逗女朋友？
    # 有哪些计算机的书适合推荐给大一学生？
    # ...


def test_column():
    url = 'http://zhuanlan.zhihu.com/xiepanda'
    column = client.column(url)

    # 获取专栏名
    print(column.name)
    # 谢熊猫出没注意

    # 获取关注人数
    print(column.follower_num)
    # 73570

    # 获取文章数量
    print(column.post_num)
    # 68

    # 获取所有文章标题
    for _, post in zip(range(0, 10), column.posts):
        print(post.title)
    # 伦敦，再见。London, Pride.
    # 为什么你来到伦敦?——没有抽到h1b
    # “城邦之国”新加坡强在哪？
    # ...


def test_post():
    url = 'http://zhuanlan.zhihu.com/xiepanda/19950456'
    post = client.post(url)

    # 获取文章地址
    print(post.url)

    # 获取文章标题
    print(post.title)
    # 为什么最近有很多名人，比如比尔盖茨，马斯克、霍金等，让人们警惕人工智能？

    # 获取所在专栏名称
    print(post.column.name)
    # 谢熊猫出没注意

    # 获取作者名称
    print(post.author.name)
    # 谢熊猫君

    # 获取赞同数
    print(post.upvote_num)
    # 18491

    # 获取评论数
    print(post.comment_num)
    # 1748

    # 保存为 markdown
    post.save(filepath='.')
    # 当前目录下生成
    # 为什么最近有很多名人，比如比尔盖茨，马斯克、霍金等，让人们警惕人工智能？ - 谢熊猫君.md


def test_topic():
    url = 'http://www.zhihu.com/topic/19947695/'
    topic = client.topic(url)
    
    # 获取话题地址
    print(topic.url)
    # http://www.zhihu.com/topic/19947695/

    # 获取话题名称
    print(topic.name)
    # 深网（Deep Web）

    # 获取话题描述信息
    print(topic.description)
    # 暗网（英语：Deep Web，又称深网、不可见网、隐藏网）

    # 获取话题头像url
    print(topic.photo_url)
    # http://pic1.zhimg.com/a3b0d77c052e45399da1fe26fb4c9734_r.jpg

    # 获取话题关注人数
    print(topic.follower_num)
    # 3309

    # 获取关注者
    for _, follower in zip(range(0, 10), topic.followers):
        print(follower.name, follower.motto)
    # 韦小宝 韦小宝
    # 吉陆遥 吉陆遥
    # qingyuan ma qingyuan ma
    # ...

    # 获取父话题
    for _, parent in zip(range(0, 10), topic.parents):
        print(parent.name)
    # 互联网

    # 获取子话题
    for _, child in zip(range(0, 10), topic.children):
        print(child.name)
    # Tor

    # 获取优秀答主
    for _, author in zip(range(0, 10), topic.top_authors):
        print(author.name, author.motto)
    # Ben Chen
    # acel rovsion 我只不过规范基点上的建构论者
    # 沈万马
    # ...

    # 获取话题下的精华回答
    for _, ans in zip(range(0, 10), topic.top_answers):
        print(ans.question.title, ans.author.name, ans.upvote_num)
    # 《纸牌屋》中提到的深网 (Deep Web) 是什么？ Ben Chen 2956
    # 黑客逃避追踪，为什么要用虚拟机 + TOR + VPN 呢？ Ario 526
    # 《纸牌屋》中提到的深网 (Deep Web) 是什么？ acel rovsion 420
    # ...

    # 按时间由近到远获取所有问题
    for _, question in zip(range(0, 10), topic.questions):
        print(question.title)
    # 马里亚纳网络存在吗？
    # 玛丽亚纳网络存在吗？
    # 为什么暗网里这么多违法的东西FBI不顺藤摸瓜呢?
    # ...

    # 按时间由近到远获取所有回答
    for _, ans in zip(range(0, 10), topic.answers):
        print(ans.question.title, ans.author.name, ans.upvote_num)
    # 如何用tor登陆qq? 匿名用户 0
    # 想看一下暗网（deep web），但是怕中病毒，所以有谁能发发截图？？ tor 0
    # icq是什么 为什么暗网交流一般都用icq？ tor 0
    # ...

    # 获取话题下的热门问题，按热门度由高到低返回
    for _, q in zip(range(0, 10), topic.hot_questions):
        print(q.title)
    # 《纸牌屋》中提到的深网 (Deep Web) 是什么？
    # 黑客逃避追踪，为什么要用虚拟机 + TOR + VPN 呢？
    # 微博热传的关于暗网的变态故事是真的还是假的啊？
    # ...

    # 获取话题下的热门回答，按热门度由高到低返回
    for _, ans in zip(range(0, 10), topic.hot_answers):
        print(ans.question.title, ans.author.name, ans.upvote_num)
    # 《纸牌屋》中提到的深网 (Deep Web) 是什么？ Ben Chen 3006
    # 《纸牌屋》中提到的深网 (Deep Web) 是什么？ 提刀夜行 123
    # 《纸牌屋》中提到的深网 (Deep Web) 是什么？ 匿名用户 21
    # ...


def test_me():
    """
    本函数默认不会开启，因为函数涉及到点赞，反对，感谢，关注等主观操作

    请确认您有能力在测试代码生错误的情况下，判断出出错在哪一行，
    并将代码进行的操作回滚（即：将点赞，感谢，关注等操作取消）

    如果确认有能力，请填写代码中的空白，并将test函数中相关行注释取消
    """
    answer = client.answer('')          # 填写答案Url，不可填写自己的答案
    post = client.post('')              # 填写文章Url，不可填写自己的文章
    author = client.author('')          # 填写用户Url，不可填写自己
    question = client.question('')      # 填写问题Url
    topic = client.topic('')            # 填写话题Url
    collection = client.collection('')  # 填写收藏夹Url

    me = client.me()

    print('赞同答案...', end='')
    assert me.vote(answer, 'up')        # 赞同
    assert me.vote(answer, 'down')      # 反对
    assert me.vote(answer, 'clear')     # 清除
    print('通过')

    print('感谢答案...', end='')
    assert me.thanks(answer)            # 感谢
    assert me.thanks(answer, False)     # 取消感谢
    print('通过')

    print('赞同文章...', end='')
    assert me.vote(post, 'up')          # 赞同
    assert me.vote(post, 'down')        # 反对
    assert me.vote(post, 'clear')       # 清除
    print('通过')

    print('关注用户...', end='')
    assert me.follow(author)            # 关注
    assert me.follow(author, False)     # 取消关注
    print('通过')

    print('关注问题...', end='')
    assert me.follow(question)          # 关注
    assert me.follow(question, False)   # 取消关注
    print('通过')

    print('关注话题...', end='')
    assert me.follow(topic)             # 关注
    assert me.follow(topic, False)      # 取消关注
    print('通过')

    print('关注收藏夹...', end='')
    assert me.follow(collection)         # 关注
    assert me.follow(collection, False)  # 取消关注
    print('通过')


def test():
    test_question()
    test_answer()
    test_author()
    test_collection()
    test_column()
    test_post()
    test_topic()
    # test_me()


if __name__ == '__main__':
    Cookies_File = 'cookies.json'
    BASE_DIR = os.path.dirname(os.path.realpath(__file__))
    TEST_DIR = os.path.join(BASE_DIR, 'test')

    print("Test dir: ", TEST_DIR)

    if os.path.exists(TEST_DIR):
        print("Cleaning it...", end='')
        shutil.rmtree(TEST_DIR)
        print("Done")
    else:
        print("Test dir not exist.")

    os.chdir(BASE_DIR)

    if os.path.isfile(Cookies_File):
        print("Cookies file found.")
        client = ZhihuClient(Cookies_File)
    else:
        print("Cookies file not exist, please login...")
        client = ZhihuClient()
        cookies_str = client.login_in_terminal()
        with open(Cookies_File, 'w') as f:
            f.write(cookies_str)

    print("Making test dir...", end="")
    os.mkdir(TEST_DIR)
    print("Done", end="\n\n")

    os.chdir(TEST_DIR)

    print("===== test start =====")

    import timeit

    try:
        time = timeit.timeit(
            'test()', setup='from __main__ import test', number=1)
        print('===== test passed =====')
        print('no error happen')
        print('time used: {0} ms'.format(time * 1000))
    except Exception as e:
        print('===== test failed =====')
        raise e
    finally:
        os.chdir(BASE_DIR)
        print("Cleaning...", end='')
        shutil.rmtree(TEST_DIR)
        print("Done")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function, division, unicode_literals
import unittest
import os
import json

from zhihu import Post
from test_utils import TEST_DATA_PATH


class ColumnTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        url = 'http://zhuanlan.zhihu.com/xiepanda/20202275'

        post_path = os.path.join(TEST_DATA_PATH, 'column_post.json')
        with open(post_path, 'r') as f:
            post_json = json.load(f)

        post_saved_path = os.path.join(TEST_DATA_PATH, 'post.md')
        with open(post_saved_path, 'rb') as f:
            cls.post_saved = f.read()

        cls.post = Post(url)
        cls.post.soup = post_json
        cls.expected = {'column_in_name': 'xiepanda', 'slug': 20202275,
                        'column_name': '谢熊猫出没注意',
                        'author_name': '谢熊猫君', 'author_id': 'xiepanda',
                        'title': '为了做一个称职的吃货，他决定连着吃一百天转基因食物',
                        'upvote_num': 963, 'comment_num': 199}

    def test_column_in_name(self):
        self.assertEqual(self.expected['column_in_name'],
                         self.post.column_in_name)

    def test_slug(self):
        self.assertEqual(self.expected['slug'], self.post.slug)

    def test_author(self):
        self.assertEqual(self.expected['author_name'], self.post.author.name)
        self.assertEqual(self.expected['author_id'], self.post.author.id)

    def test_title(self):
        self.assertEqual(self.expected['title'], self.post.title)

    def test_upvote_num(self):
        self.assertEqual(self.expected['upvote_num'], self.post.upvote_num)

    def test_comment_num(self):
        self.assertEqual(self.expected['comment_num'], self.post.comment_num)

    def test_save(self):
        save_name = 'post_save'
        self.post.save(filepath=TEST_DATA_PATH, filename=save_name)
        post_saved_path = os.path.join(TEST_DATA_PATH, save_name + '.md')
        with open(post_saved_path, 'rb') as f:
            post_saved = f.read()
        os.remove(post_saved_path)
        self.assertEqual(self.post_saved, post_saved)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json

from .common import *


class Question:

    """问题类，请使用``ZhihuClient.question``方法构造对象."""

    @class_common_init(re_question_url)
    def __init__(self, url, title=None, followers_num=None,
                 answer_num=None, session=None):
        """创建问题类实例.

        :param str url: 问题url
        :param str title: 问题标题，可选,
        :param int followers_num: 问题关注人数，可选
        :param int answer_num: 问题答案数，可选
        :return: 问题对象
        :rtype: Question
        """
        self._session = session
        self.url = url
        self._title = title
        self._answer_num = answer_num
        self._followers_num = followers_num
        self._id = int(re.match(r'.*/(\d+)', self.url).group(1))

    def _make_soup(self):
        if self.soup is None:
            r = self._session.get(self.url)
            self.soup = BeautifulSoup(r.content)

    @property
    def id(self):
        """获取问题id（网址最后的部分）.

        :return: 问题id
        :rtype: int
        """
        return self._id

    @property
    @check_soup('_qid')
    def qid(self):
        """获取问题内部id（用不到就忽视吧）

        :return: 问题内部id
        :rtype: int
        """
        return int(self.soup.find(
            'div', id='zh-question-detail')['data-resourceid'])

    @property
    @check_soup('_xsrf')
    def xsrf(self):
        """获取知乎的反xsrf参数（用不到就忽视吧~）

        :return: xsrf参数
        :rtype: str
        """
        return self.soup.find(
            'input', attrs={'name': '_xsrf'})['value']

    @property
    @check_soup('_html')
    def html(self):
        """获取页面源码.

        :return: 页面源码
        :rtype: str
        """
        return self.soup.prettify()

    @property
    @check_soup('_title')
    def title(self):
        """获取问题标题.

        :return: 问题标题
        :rtype: str
        """
        return self.soup.find('h2', class_='zm-item-title') \
            .text.replace('\n', '')

    @property
    @check_soup('_details')
    def details(self):
        """获取问题详细描述，目前实现方法只是直接获取文本，效果不满意……等更新.

        :return: 问题详细描述
        :rtype: str
        """
        return self.soup.find("div", id="zh-question-detail").div.text

    @property
    @check_soup('_answers_num')
    def answer_num(self):
        """获取问题答案数量.

        :return: 问题答案数量
        :rtype: int
        """
        answer_num_block = self.soup.find('h3', id='zh-question-answer-num')
        # 当0人回答或1回答时，都会找不到 answer_num_block，
        # 通过找答案的赞同数block来判断到底有没有答案。
        # （感谢知乎用户 段晓晨 提出此问题）
        if answer_num_block is None:
            if self.soup.find('span', class_='count') is not None:
                return 1
            else:
                return 0
        return int(answer_num_block['data-num'])

    @property
    @check_soup('_follower_num')
    def follower_num(self):
        """获取问题关注人数.

        :return: 问题关注人数
        :rtype: int
        """
        follower_num_block = self.soup.find('div', class_='zg-gray-normal')
        # 无人关注时 找不到对应block，直接返回0 （感谢知乎用户 段晓晨 提出此问题）
        if follower_num_block.strong is None:
            return 0
        return int(follower_num_block.strong.text)

    @property
    @check_soup('_topics')
    def topics(self):
        """获取问题所属话题.

        :return: 问题所属话题
        :rtype: list(str)
        """
        topics_list = []
        for topic in self.soup.find_all('a', class_='zm-item-tag'):
            topics_list.append(topic.text.replace('\n', ''))
        return topics_list

    @property
    def followers(self):
        """获取关注此问题的用户

        :return: 关注此问题的用户
        :rtype: Author.Iterable
        :问题: 要注意若执行过程中另外有人关注，可能造成重复获取到某些用户
        """
        self._make_soup()
        followers_url = self.url + 'followers'
        for x in common_follower(followers_url, self.xsrf, self._session):
            yield x

    @property
    def answers(self):
        """获取问题的所有答案.

        :return: 问题的所有答案，返回生成器
        :rtype: Answer.Iterable
        """
        from .author import Author
        from .answer import Answer

        self._make_soup()
        new_header = dict(Default_Header)
        new_header['Referer'] = self.url
        params = {"url_token": self.id,
                  'pagesize': '50',
                  'offset': 0}
        data = {'_xsrf': self.xsrf,
                'method': 'next',
                'params': ''}
        for i in range(0, (self.answer_num - 1) // 50 + 1):
            if i == 0:
                # 修正各种建议修改的回答……
                error_answers = self.soup.find_all('div', id='answer-status')
                for each in error_answers:
                    each['class'] = 'zm-editable-content'
                # 正式处理
                authors = self.soup.find_all(
                    'h3', class_='zm-item-answer-author-wrap')
                urls = self.soup.find_all('a', class_='answer-date-link')
                upvote_nums = self.soup.find_all('div',
                                                 class_='zm-item-vote-info')
                contents = self.soup.find_all(
                    'div', class_='zm-editable-content')
                for author, url, upvote_num, content in \
                        zip(authors, urls, upvote_nums, contents):
                    a_url, name, motto, photo = parser_author_from_tag(author)
                    author_obj = Author(a_url, name, motto, photo_url=photo,
                                        session=self._session)
                    url = Zhihu_URL + url['href']
                    upvote_num = int(upvote_num['data-votecount'])
                    content = answer_content_process(content)
                    yield Answer(url, self, author_obj, upvote_num, content,
                                 session=self._session)
            else:
                params['offset'] = i * 50
                data['params'] = json.dumps(params)
                r = self._session.post(Question_Get_More_Answer_URL, data=data,
                                       headers=new_header)
                answer_list = r.json()['msg']
                for answer_html in answer_list:
                    yield self._parse_answer_html(answer_html, Author, Answer)

    @property
    def top_answer(self):
        """获取排名第一的答案.

        :return: 排名第一的答案
        :rtype: Answer
        """
        for a in self.answers:
            return a

    def top_i_answer(self, i):
        """获取排名某一位的答案.

        :param int i: 要获取的答案的排名
        :return: 答案对象，能直接获取的属性参见answers方法
        :rtype: Answer
        """
        for j, a in enumerate(self.answers):
            if j == i - 1:
                return a

    def top_i_answers(self, i):
        """获取排名在前几位的答案.

        :param int i: 获取前几个
        :return: 答案对象，返回生成器
        :rtype: Answer.Iterable
        """
        for j, a in enumerate(self.answers):
            if j <= i - 1:
                yield a
            else:
                return

    def _parse_answer_html(self, answer_html, Author, Answer):
        soup = BeautifulSoup(answer_html)
        # 修正各种建议修改的回答……
        error_answers = soup.find_all('div', id='answer-status')
        for each in error_answers:
            each['class'] = 'zm-editable-content'
        answer_url = \
            self.url + 'answer/' + soup.div['data-atoken']
        author = soup.find(
            'h3', class_='zm-item-answer-author-wrap')
        upvote_num = int(soup.find(
            'div', class_='zm-item-vote-info')['data-votecount'])
        content = soup.find(
            'div', class_='zm-editable-content')
        content = answer_content_process(content)
        a_url, name, motto, photo = parser_author_from_tag(author)
        author_obj = Author(a_url, name, motto, photo_url=photo,
                            session=self._session)
        return Answer(answer_url, self, author_obj,
                      upvote_num, content, session=self._session)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import datetime

from .common import *


class Author:

    """用户类，请使用``ZhihuClient.answer``方法构造对象."""

    @class_common_init(re_author_url, True)
    def __init__(self, url, name=None, motto=None, follower_num=None,
                 question_num=None, answer_num=None, upvote_num=None,
                 thank_num=None, photo_url=None, session=None):
        """创建用户类实例.

        :param str url: 用户主页url，形如 http://www.zhihu.com/people/7sdream
        :param str name: 用户名字，可选
        :param str motto: 用户简介，可选
        :param int follower_num: 用户粉丝数，可选
        :param int question_num: 用户提问数，可选
        :param int answer_num: 用户答案数，可选
        :param int upvote_num: 用户获得赞同数，可选
        :param int thank_num: 用户获得感谢数，可选
        :param str photo_url: 用户头像地址，可选
        :param Session session: 使用的网络会话，为空则使用新会话。
        :return: 用户对象
        :rtype: Author
        """
        self.url = url
        self._session = session
        self.card = None
        self._nav_list = None
        self._name = name
        self._motto = motto
        self._follower_num = follower_num
        self._question_num = question_num
        self._answer_num = answer_num
        self._upvote_num = upvote_num
        self._thank_num = thank_num
        self._photo_url = photo_url

    def _make_soup(self):
        if self.soup is None and self.url is not None:
            r = self._session.get(self.url)
            self.soup = BeautifulSoup(r.content)
            self._nav_list = self.soup.find(
                'div', class_='profile-navbar').find_all('a')

    def _make_card(self):
        if self.card is None and self.url is not None:
            params = {'url_token': self.id}
            real_params = {'params': json.dumps(params)}
            r = self._session.get(Get_Profile_Card_URL, params=real_params)
            self.card = BeautifulSoup(r.content)

    @property
    def id(self):
        """获取用户id，就是网址最后那一部分.

        :return: 用户id
        :rtype: str
        """
        return re.match(r'^.*/([^/]+)/$', self.url).group(1) if self.url is not None else ''

    @property
    @check_soup('_xsrf')
    def xsrf(self):
        """获取知乎的反xsrf参数（用不到就忽视吧~）

        :return: xsrf参数
        :rtype: str
        """
        self._make_soup()
        return self.soup.find(
            'input', attrs={'name': '_xsrf'})['value']

    @property
    @check_soup('_hash_id')
    def hash_id(self):
        """获取作者的内部hash id（用不到就忽视吧~）

        :return: 用户hash id
        :rtype: str
        """
        div = self.soup.find('div', class_='zm-profile-header-op-btns')
        if div is not None:
            return div.button['data-id']
        else:
            ga = self.soup.find('script', attrs={'data-name': 'ga_vars'})
            return json.loads(ga.text)['user_hash']

    @property
    @check_soup('_name', '_make_card')
    def name(self):
        """获取用户名字.

        :return: 用户名字
        :rtype: str
        """
        if self.url is None:
            return '匿名用户'
        if self.soup is not None:
            return self.soup.find('div', class_='title-section').span.text
        else:
            assert self.card is not None
            return self.card.find('span', class_='name').text

    @property
    @check_soup('_motto', '_make_card')
    def motto(self):
        """获取用户自我介绍，由于历史原因，我还是把这个属性叫做motto吧.

        :return: 用户自我介绍
        :rtype: str
        """
        if self.url is None:
            return ''
        else:
            if self.soup is not None:
                bar = self.soup.find(
                    'div', class_='title-section')
                if len(bar.contents) < 4:
                    return ''
                else:
                    return bar.contents[3].text
            else:
                assert self.card is not None
                motto = self.card.find('div', class_='tagline')
                return motto.text if motto is not None else ''

    @property
    @check_soup('_photo_url', '_make_card')
    def photo_url(self):
        """获取用户头像图片地址.

        :return: 用户头像url
        :rtype: str
        """
        if self.url is not None:
            if self.soup is not None:
                img = 'http:' + self.soup.find(
                    'img', class_='avatar avatar-l')['src']
                return img.replace('_l', '_r')
            else:
                assert(self.card is not None)
                return 'http:' + self.card.img['src'].replace('_xs', '_r')
        else:
            return 'http://pic1.zhimg.com/da8e974dc_r.jpg'

    @property
    @check_soup('_followee_num')
    def followee_num(self):
        """获取关注了多少人.

        :return: 关注的人数
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'div', class_='zm-profile-side-following').a.strong.text)
            return number

    @property
    @check_soup('_follower_num')
    def follower_num(self):
        """获取追随者数量，就是关注此人的人数.

        :return: 追随者数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'div', class_='zm-profile-side-following zg-clear').find_all(
                'a')[1].strong.text)
            return number

    @property
    @check_soup('_upvote_num')
    def upvote_num(self):
        """获取收到的的赞同数量.

        :return: 收到的的赞同数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'span', class_='zm-profile-header-user-agree').strong.text)
            return number

    @property
    @check_soup('_thank_num')
    def thank_num(self):
        """获取收到的感谢数量.

        :return: 收到的感谢数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'span', class_='zm-profile-header-user-thanks').strong.text)
            return number

    @property
    @check_soup('_question_num')
    def question_num(self):
        """获取提问数量.

        :return: 提问数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[1].span.text)

    @property
    @check_soup('_answer_num')
    def answer_num(self):
        """获取答案数量.

        :return: 答案数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[2].span.text)

    @property
    @check_soup('_post_num')
    def post_num(self):
        """获取专栏文章数量.

        :return: 专栏文章数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[3].span.text)

    @property
    @check_soup('_collection_num')
    def collection_num(self):
        """获取收藏夹数量.

        :return: 收藏夹数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[4].span.text)

    @property
    @check_soup('_followed_column_num')
    def followed_column_num(self):
        """获取用户关注的专栏数

        :return: 关注的专栏数
        :rtype: int
        """
        if self.url is not None:
            tag = self.soup.find('div', class_='zm-profile-side-columns')
            if tag is not None:
                return int(re_get_number.match(tag.parent.strong.text).group(1))
        return 0

    @property
    def questions(self):
        """获取用户的所有问题.

        :return: 用户的所有问题，返回生成器.
        :rtype: Question.Iterable
        """
        from .question import Question

        if self.url is None or self.question_num == 0:
            return
        for page_index in range(1, (self.question_num - 1) // 20 + 2):
            html = self._session.get(
                self.url + 'asks?page=' + str(page_index)).text
            soup = BeautifulSoup(html)
            question_links = soup.find_all('a', class_='question_link')
            question_datas = soup.find_all(
                'div', class_='zm-profile-section-main')
            for link, data in zip(question_links, question_datas):
                url = Zhihu_URL + link['href']
                title = link.text
                answer_num = int(
                    re_get_number.match(data.div.contents[4]).group(1))
                follower_num = int(
                    re_get_number.match(data.div.contents[6]).group(1))
                q = Question(url, title, follower_num, answer_num,
                             session=self._session)
                yield q

    @property
    def answers(self):
        """获取用户的所有答案.

        :return: 用户所有答案，返回生成器.
        :rtype: Answer.Iterable
        """
        from .question import Question
        from .answer import Answer

        if self.url is None or self.answer_num == 0:
            return
        for page_index in range(1, (self.answer_num - 1) // 20 + 2):
            html = self._session.get(
                self.url + 'answers?page=' + str(page_index)).text
            soup = BeautifulSoup(html)
            questions = soup.find_all('a', class_='question_link')
            upvotes = soup.find_all('a', class_='zm-item-vote-count')
            for q, upvote in zip(questions, upvotes):
                answer_url = Zhihu_URL + q['href']
                question_url = Zhihu_URL + re_a2q.match(q['href']).group(1)
                question_title = q.text
                upvote = int(upvote['data-votecount'])
                question = Question(question_url, question_title,
                                    session=self._session)
                yield Answer(answer_url, question, self, upvote,
                             session=self._session)

    @property
    def followers(self):
        """获取关注此用户的人.

        :return: 关注此用户的人，返回生成器
        :rtype: Author.Iterable
        """
        for x in self._follow_ee_ers('er'):
            yield x

    @property
    def followees(self):
        """获取用户关注的人.

        :return: 用户关注的人的，返回生成器
        :rtype: Author.Iterable
        """
        for x in self._follow_ee_ers('ee'):
            yield x

    def _follow_ee_ers(self, t):
        if self.url is None:
            return
        if t == 'er':
            request_url = Author_Get_More_Followers_URL
        else:
            request_url = Author_Get_More_Followees_URL
        self._make_card()
        if self.hash_id is None:
            self._make_soup()
        headers = dict(Default_Header)
        headers['Referer'] = self.url + 'follow' + t + 's'
        params = {"order_by": "created", "offset": 0, "hash_id": self.hash_id}
        data = {'_xsrf': self.xsrf, 'method': 'next', 'params': ''}
        gotten_date_num = 20
        offset = 0
        while gotten_date_num == 20:
            params['offset'] = offset
            data['params'] = json.dumps(params)
            res = self._session.post(request_url, data=data, headers=headers)
            json_data = res.json()
            gotten_date_num = len(json_data['msg'])
            offset += gotten_date_num
            for html in json_data['msg']:
                soup = BeautifulSoup(html)
                h2 = soup.find('h2')
                author_name = h2.a.text
                author_url = h2.a['href']
                author_motto = soup.find('div', class_='zg-big-gray').text
                author_photo = PROTOCOL + soup.a.img['src'].replace('_m', '_r')
                numbers = [int(re_get_number.match(x.text).group(1))
                           for x in soup.find_all('a', target='_blank')]
                yield Author(author_url, author_name, author_motto, *numbers,
                             photo_url=author_photo, session=self._session)

    @property
    def collections(self):
        """获取用户收藏夹.

        :return: 用户收藏夹，返回生成器
        :rtype: Collection.Iterable
        """
        from .collection import Collection

        if self.url is None or self.collection_num == 0:
            return
        else:
            collection_num = self.collection_num
            for page_index in range(1, (collection_num - 1) // 20 + 2):
                html = self._session.get(
                    self.url + 'collections?page=' + str(page_index)).text
                soup = BeautifulSoup(html)
                collections_names = soup.find_all(
                    'a', class_='zm-profile-fav-item-title')
                collection_follower_nums = soup.find_all(
                    'div', class_='zm-profile-fav-bio')
                for c, f in zip(collections_names, collection_follower_nums):
                    c_url = Zhihu_URL + c['href']
                    c_name = c.text
                    c_fn = int(re_get_number.match(f.contents[2]).group(1))
                    yield Collection(c_url, self, c_name, c_fn,
                                     session=self._session)

    @property
    def columns(self):
        """获取用户专栏.

        :return: 用户专栏，返回生成器
        :rtype: Column.Iterable
        """
        from .column import Column

        if self.url is None or self.post_num == 0:
            return
        soup = BeautifulSoup(self._session.get(self.url + 'posts').text)
        column_tags = soup.find_all('div', class_='column')
        for column_tag in column_tags:
            name = column_tag.div.a.span.text
            url = column_tag.div.a['href']
            follower_num = int(re_get_number.match(
                column_tag.div.div.a.text).group(1))
            footer = column_tag.find('div', class_='footer')
            if footer is None:
                post_num = 0
            else:
                post_num = int(
                    re_get_number.match(footer.a.text).group(1))
            yield Column(url, name, follower_num, post_num,
                         session=self._session)

    @property
    def followed_columns(self):
        """获取用户关注的专栏.

        :return: 用户关注的专栏，返回生成器
        :rtype: Column.Iterable
        """
        from .column import Column
        if self.url is None:
            return
        if self.followed_column_num > 0:
            tag = self.soup.find('div', class_='zm-profile-side-columns')
            if tag is not None:
                for a in tag.find_all('a'):
                    yield Column(a['href'], a.img['alt'])
            if self.followed_column_num > 7:
                offset = 7
                gotten_data_num = 20
                while gotten_data_num == 20:
                    params = {'hash_id': self.hash_id, 'limit': 20, 'offset': offset}
                    data = {'method': 'next', '_xsrf': self.xsrf, 'params': json.dumps(params)}
                    j = self._session.post(Author_Get_More_Follow_Column_URL, data=data).json()
                    gotten_data_num = len(j['msg'])
                    offset += gotten_data_num
                    for msg in map(BeautifulSoup, j['msg']):
                        name = msg.strong.text
                        url = msg.a['href']
                        post_num = int(re_get_number.match(msg.span.text).group(1))
                        yield Column(url, name, post_num)

    @property
    def activities(self):
        """获取用户的最近动态.

        :return: 最近动态，返回生成器，具体说明见 :class:`.Activity`
        :rtype: Activity.Iterable
        """
        from .activity import Activity
        from .acttype import ActType
        from .question import Question
        from .answer import Answer
        from .column import Column
        from .post import Post
        from .topic import Topic

        if self.url is None:
            return
        self._make_soup()
        gotten_feed_num = 20
        start = '0'
        while gotten_feed_num == 20:
            data = {'_xsrf': self.xsrf, 'start': start}
            res = self._session.post(self.url + 'activities', data=data)
            gotten_feed_num = res.json()['msg'][0]
            soup = BeautifulSoup(res.json()['msg'][1])
            acts = soup.find_all(
                'div', class_='zm-profile-section-item zm-item clearfix')
            start = acts[-1]['data-time'] if len(acts) > 0 else 0
            for act in acts:
                act_time = datetime.datetime.fromtimestamp(
                    int(act['data-time']))
                useless_tag = act.div.find('a', class_='zg-link')
                if useless_tag is not None:
                    useless_tag.extract()
                type_string = next(act.div.stripped_strings)
                if type_string in ['赞同了', '在']:    # 赞同文章 or 发表文章
                    act_type = ActType.UPVOTE_POST \
                        if type_string == '赞同了' else ActType.PUBLISH_POST

                    column_url = act.find('a', class_='column_link')['href']
                    column_name = act.find('a', class_='column_link').text
                    column = Column(column_url, column_name,
                                    session=self._session)
                    try:
                        author_tag = act.find('div', class_='author-info')
                        author_url = Zhihu_URL + author_tag.a['href']
                        author_info = list(author_tag.stripped_strings)
                        author_name = author_info[0]
                        author_motto = author_info[1] \
                            if len(author_info) > 1 else ''
                    except TypeError:
                        author_url = None
                        author_name = '匿名用户'
                        author_motto = ''
                    author = Author(author_url, author_name, author_motto,
                                    session=self._session)
                    post_url = act.find('a', class_='post-link')['href']
                    post_title = act.find('a', class_='post-link').text
                    post_comment_num, post_upvote_num = self._parse_un_cn(act)
                    post = Post(post_url, column, author, post_title,
                                post_upvote_num, post_comment_num,
                                session=self._session)

                    yield Activity(act_type, act_time, post=post)
                elif type_string == '关注了专栏':
                    act_type = ActType.FOLLOW_COLUMN
                    column = Column(act.div.a['href'], act.div.a.text,
                                    session=self._session)
                    yield Activity(act_type, act_time, column=column)
                elif type_string == '关注了问题':
                    act_type = ActType.FOLLOW_QUESTION
                    question = Question(Zhihu_URL + act.div.a['href'],
                                        act.div.a.text, session=self._session)
                    yield Activity(act_type, act_time, question=question)
                elif type_string == '提了一个问题':
                    act_type = ActType.ASK_QUESTION
                    question = Question(Zhihu_URL +
                                        act.div.contents[3]['href'],
                                        list(act.div.children)[3].text,
                                        session=self._session)
                    yield Activity(act_type, act_time, question=question)
                elif type_string == '赞同了回答':
                    act_type = ActType.UPVOTE_ANSWER
                    question_url = Zhihu_URL + re_a2q.match(
                        act.div.a['href']).group(1)
                    question_title = act.div.a.text
                    question = Question(question_url, question_title,
                                        session=self._session)

                    try_find_author = act.find('h3').find_all(
                        'a', href=re.compile('^/people/[^/]*$'))
                    if len(try_find_author) == 0:
                        author_url = None
                        author_name = '匿名用户'
                        author_motto = ''
                        photo_url = None
                    else:
                        try_find_author = try_find_author[-1]
                        author_url = Zhihu_URL + try_find_author['href']
                        author_name = try_find_author.text
                        try_find_motto = try_find_author.parent.strong
                        if try_find_motto is None:
                            author_motto = ''
                        else:
                            author_motto = try_find_motto['title']
                        photo_url = PROTOCOL + try_find_author.parent.a.img[
                            'src'].replace('_s', '_r')
                    author = Author(author_url, author_name, author_motto,
                                    photo_url=photo_url, session=self._session)

                    answer_url = Zhihu_URL + act.div.a['href']
                    answer_comment_num, answer_upvote_num = \
                        self._parse_un_cn(act)
                    answer = Answer(answer_url, question, author,
                                    answer_upvote_num, session=self._session)

                    yield Activity(act_type, act_time, answer=answer)
                elif type_string == '回答了问题':
                    act_type = ActType.ANSWER_QUESTION
                    question_url = Zhihu_URL + re_a2q.match(
                        act.div.find_all('a')[-1]['href']).group(1)
                    question_title = act.div.find_all('a')[-1].text
                    question = Question(question_url, question_title,
                                        session=self._session)

                    answer_url = Zhihu_URL + \
                        act.div.find_all('a')[-1]['href']
                    answer_comment_num, answer_upvote_num = \
                        self._parse_un_cn(act)
                    answer = Answer(answer_url, question, self,
                                    answer_upvote_num, session=self._session)

                    yield Activity(act_type, act_time, answer=answer)
                elif type_string == '关注了话题':
                    act_type = ActType.FOLLOW_TOPIC
                    topic_url = Zhihu_URL + act.div.a['href']
                    topic_name = act.div.a['title']
                    topic = Topic(topic_url, topic_name, session=self._session)
                    yield Activity(act_type, act_time, topic=topic)

    def is_zero_user(self):
        """返回当前用户是否为三零用户，其实是四零： 赞同0，感谢0，提问0，回答0.

        :return: 是否是三零用户
        :rtype: bool
        """
        return self.upvote_num + self.thank_num + \
            self.question_num + self.answer_num == 0

    @staticmethod
    def _parse_un_cn(act):
        upvote_num = int(act.find(
            'a', class_='zm-item-vote-count')['data-votecount'])
        comment = act.find('a', class_='toggle-comment')
        comment_text = next(comment.stripped_strings)
        comment_num_match = re_get_number.match(comment_text)
        comment_num = int(comment_num_match.group(1)) \
            if comment_num_match is not None else 0
        return comment_num, upvote_num



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import enum


class ActType(enum.Enum):

    """用于表示用户动态的类型.

    :常量说明:
        =============== ============== ======== ==================
        常量名          说明           提供属性 属性类型
        =============== ============== ======== ==================
        ANSWER_QUESTION 回答了一个问题 answer   :class:`.Answer`
        UPVOTE_ANSWER   赞同了一个回答 answer   :class:`.Answer`
        ASK_QUESTION    提出了一个问题 question :class:`.Question`
        FOLLOW_QUESTION 关注了一个问题 question :class:`.Question`
        UPVOTE_POST     赞同了一篇文章 post     :class:`.Post`
        FOLLOW_COLUMN   关注了一个专栏 column   :class:`.Column`
        FOLLOW_TOPIC    关注了一个话题 topic    :class:`.Topic`
        PUBLISH_POST    发表了一篇文章 post     :class:`.Post`
        =============== ============== ======== ==================

    """

    ANSWER_QUESTION = 1
    UPVOTE_ANSWER = 2
    ASK_QUESTION = 4
    FOLLOW_QUESTION = 8
    UPVOTE_POST = 16
    FOLLOW_COLUMN = 32
    FOLLOW_TOPIC = 64
    PUBLISH_POST = 128



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import *


class Post:

    """专栏文章类，请使用``ZhihuClient.post``方法构造对象."""

    @class_common_init(re_post_url)
    def __init__(self, url, column=None, author=None, title=None,
                 upvote_num=None, comment_num=None, session=None):
        """创建专栏文章类实例.

        :param str url: 文章url
        :param Column column: 文章所属专栏，可选
        :param Author author: 文章作者，可选
        :param str title: 文章标题，可选
        :param int upvote_num: 文章赞同数，可选
        :param int comment_num: 文章评论数，可选
        :param Session session: 使用的网络会话，为空则使用新会话
        :return: 专栏文章对象
        :rtype: Post
        """
        match = re_post_url.match(url)
        self.url = url
        self._session = session
        self._column = column
        self._author = author
        self._title = title
        self._upvote_num = upvote_num
        self._comment_num = comment_num
        self._column_in_name = match.group(1)    # 专栏内部名称
        self._slug = int(match.group(2))  # 文章编号

    def _make_soup(self):
        if self.soup is None:
            origin_host = self._session.headers.get('Host')
            self._session.headers.update(Host='zhuanlan.zhihu.com')
            self.soup = self._session.get(
                Column_Post_Data.format(
                    self.column_in_name, self.slug)).json()
            self._session.headers.update(Host=origin_host)

    @property
    def column_in_name(self):
        """获取文章所在专栏的内部名称（用不到就忽视吧~）

        :return: 专栏的内部名称
        :rtype: str
        """
        return self._column_in_name

    @property
    def slug(self):
        """获取文章的编号（用不到就忽视吧~）

        :return: 文章编号
        :rtype: int
        """
        return self._slug

    @property
    @check_soup('_column')
    def column(self):
        """获取文章所在专栏.

        :return: 文章所在专栏
        :rtype: Column
        """
        from .column import Column

        url = Column_Url + '/' + self.soup['column']['slug']
        name = self.soup['column']['name']
        return Column(url, name, session=self._session)

    @property
    @check_soup('_author')
    def author(self):
        """获取文章作者.

        :return: 文章作者
        :rtype: Author
        """
        from .author import Author

        url = self.soup['author']['profileUrl']
        name = self.soup['author']['name']
        motto = self.soup['author']['bio']
        template = self.soup['author']['avatar']['template']
        photo_id = self.soup['author']['avatar']['id']
        photo_url = template.format(id=photo_id, size='r')
        return Author(url, name, motto, photo_url=photo_url,
                      session=self._session)

    @property
    @check_soup('_title')
    def title(self):
        """获取文章标题.

        :return: 文章标题
        :rtype: str
        """
        return self.soup['title']

    @property
    @check_soup('_upvote_num')
    def upvote_num(self):
        """获取文章赞同数量.

        :return: 文章赞同数
        :rtype: int
        """
        return int(self.soup['likesCount'])

    @property
    @check_soup('_comment_num')
    def comment_num(self):
        """获取评论数量.

        :return: 评论数量
        :rtype: int
        """
        return self.soup['commentsCount']

    def save(self, filepath=None, filename=None):
        """将文章保存为 markdown 格式.

        :param str filepath: 要保存的文件所在的绝对目录或相对目录，
            不填为当前目录下以专栏名命名的目录, 设为"."则为当前目录
        :param str filename: 要保存的文件名，不填则默认为 文章标题 - 作者名.md
            如果文件已存在，自动在后面加上数字区分。
            自定义参数时请不要输入扩展名 .md
        :return:
        """
        self._make_soup()
        file = get_path(filepath, filename, 'md',
                        self.column.name,
                        self.title + ' - ' + self.author.name)
        with open(file, 'wb') as f:
            import html2text
            h2t = html2text.HTML2Text()
            h2t.body_width = 0
            f.write(h2t.handle(self.soup['content']).encode('utf-8'))
